﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GasolineTanker
{
    internal class SetGasolineTankerGeneric<T>
        where T : class, IEquatable<T>
    {
		private readonly List<T> _places;
		public int Count => _places.Count;
		private readonly int _maxCount;
		public SetGasolineTankerGeneric(int count)
		{
			_maxCount = count;
			_places = new List<T>();
		}
		public int Insert(T gasolineTanker)
		{
            return Insert(gasolineTanker, 0);
        }
        private bool isCorrectPosition(int position)
        {
            return 0 <= position && position < _maxCount;
        }
        public int Insert(T gasolineTanker, int position)
		{
            if (Count == _maxCount)
                throw new StorageOverflowException(_maxCount);

            if (position < 0 || position > Count)
                return -1;

            if (_places.Contains(gasolineTanker))
                return -1;

            _places.Insert(position, gasolineTanker);

            return position;
        }
		public T Remove(int position)
		{
            if (!isCorrectPosition(position))
                return null;
            var result = this[position];
            if (result == null)
                throw new GasolineTankerNotFoundException(position);
            _places.RemoveAt(position);
            return result;
        }
        public T this[int position]
        {
            get
            {
                if (position < 0 || position >= _places.Count) return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position >= _places.Count) return;
                _places.Insert(position, value);
            }
        }
        public IEnumerable<T> GetGasolineTanker()
        {
            foreach (var gasolineTanker in _places)
            {
                if (gasolineTanker != null)
                {
                    yield return gasolineTanker;
                }
                else
                {
                    yield break;
                }
            }
        }
        public void SortSet(IComparer<T> comparer)
        {
            if (comparer == null)
            {
                return;
            }
            _places.Sort(comparer);
        }
    }
}
